@@ -8,13 +8,13 @@
 # executed in 'sh', not in 'tclsh' \
 exec tclsh "$0" "$@"
 
-package provide install_pfm 1.2.3
+package provide install_pfm 1.5.4
 
 package require Tk
 
 namespace eval install {
 
-    variable API "pgintcl 3.0.0"
+    variable API "pgintcl 3.0.1"
     variable TRE "Tclkit"
     variable apiHelp
     variable treHelp
@@ -23,30 +23,30 @@ namespace eval install {
     variable launchDir {~/bin}
     global env
 
-    set apiHelp "pgintcl 3.0.0 requires postgreSQL 7.4 or higher.\n"
-    set apiHelp "$apiHelp\npgintcl 1.5.0 requires postgreSQL 6.4 or higher.\n"
-    set apiHelp "$apiHelp\npgintcl 1.5.0 only supports charcter encoding LATIN1.\n"
-    set apiHelp "$apiHelp\nPgtcl has better performance, but is not included"
-    set apiHelp "$apiHelp\nin this distribution.\n"
-    set apiHelp "$apiHelp\nIf you want to use Pgtcl, you have to download it from\n"
-    set apiHelp "$apiHelp\nhttp://gborg.postgresql.org/project/pgtclng/projdisplay.php\n"
-    set apiHelp "$apiHelp\nPgtcl cannot be used when pfm runs under the Tclkit."
+    set apiHelp "pgintcl 3.0.1 requires postgreSQL 7.4 or higher.\n\n"
+    append apiHelp "pgintcl 1.5.0 requires postgreSQL 6.4 or higher.\n\n"
+    append apiHelp "pgintcl 1.5.0 only supports charcter encoding LATIN1.\n\n"
+    append apiHelp "Pgtcl has better performance, but is not included\n"
+    append apiHelp "in this distribution.\n\n"
+    append apiHelp "If you want to use Pgtcl, you have to download it from\n\n"
+    append apiHelp "http://gborg.postgresql.org/project/pgtclng/projdisplay.php\n\n"
+    append apiHelp "Pgtcl cannot be used when pfm runs under the Tclkit."
 
-    set treHelp "If Tcl/Tk version 8.4 is installed on your system,"
-    set treHelp "$treHelp\nyou can run pfm in the Tcl/Tk runtime environenment.\n"
-    set treHelp "$treHelp\nYou can verify the presence of Tcl/Tk as follows.\n"
-    set treHelp "$treHelp\nFrom the command prompt type 'tclsh'."
-    set treHelp "$treHelp\nYou should get a '%' prompt.\n"
-    set treHelp "$treHelp\nTo see the version, type 'info tclversion'"
-    set treHelp "$treHelp\nat the '%' prompt. You should get 8.4.\n"
-    set treHelp "$treHelp\nThen type 'package require Tk"
-    set treHelp "$treHelp\nYou should get 8.4 and an empty window titled 'tclsh'\n"
-    set treHelp "$treHelp\nElse, you can use the Tclkit included in the"
-    set treHelp "$treHelp\npfm distribution.\n"
-    set treHelp "$treHelp\nUsing the Tclkit excludes the use of Pgtcl."
+    set treHelp "If Tcl/Tk version 8.4 is installed on your system,\n"
+    append treHelp "you can run pfm in the Tcl/Tk runtime environenment.\n\n"
+    append treHelp "You can verify the presence of Tcl/Tk as follows.\n\n"
+    append treHelp "From the command prompt type 'tclsh'.\n"
+    append treHelp "You should get a '%' prompt.\n\n"
+    append treHelp "To see the version, type 'info tclversion'\n"
+    append treHelp "at the '%' prompt. You should get 8.4.\n\n"
+    append treHelp "Then type 'package require Tk\n"
+    append treHelp "You should get 8.4 and an empty window titled 'tclsh'\n\n"
+    append treHelp "Else, you can use the Tclkit included in the\n"
+    append treHelp "pfm distribution.\n\n"
+    append treHelp "Using the Tclkit excludes the use of Pgtcl."
 
-    set dirHelp "Where do you want to install the pfm launch script?"
-    set dirHelp "$dirHelp\nPreferrably take one from PATH."
+    set dirHelp "Where do you want to install the pfm launch script?\n"
+    append dirHelp "Preferrably take one from PATH."
 
     set pathList [split $env(PATH) :]
 
@@ -77,8 +77,8 @@ namespace eval install {
 	place .api.title -x 0 -y 5 -relx 0.5 -anchor n
 	frame .api.buttons -relief sunken -borderwidth 2
 	place .api.buttons -x 0 -y 30 -relwidth 0.2 -relheight 1 -height -30
- 	radiobutton .api.buttons.rb201 -variable ::install::API -value "pgintcl 3.0.0" \
- 	    -text "pgintcl 3.0.0" -anchor w -command {
+ 	radiobutton .api.buttons.rb201 -variable ::install::API -value "pgintcl 3.0.1" \
+ 	    -text "pgintcl 3.0.1" -anchor w -command {
 		.tre.buttons.rbkit configure -state normal
 	    }
  	radiobutton .api.buttons.rb150 -variable ::install::API -value "pgintcl 1.5.0" \
@@ -212,15 +212,15 @@ namespace eval install {
 		writeToLog "mkdir $launchDir\n\n"
 	    }
 	}
-	set script "\#!/bin/sh"
-	set script "$script\n# pfm launch script, generated by $pfmDir/install.tcl\n"
+	set script "\#!/bin/sh\n"
+	append script "# pfm launch script, generated by $pfmDir/install.tcl\n\n"
 	switch $TRE {
 	    "Tcl/Tk" {
-		set script "$script\ntclsh [file join $pfmDir pfm.tcl] $pfmDir\n"
+		append script "tclsh [file join $pfmDir pfm.tcl] $pfmDir\n"
 	    }
 	    "Tclkit" {
-		set script "$script\n[file join $pfmDir tclkit tclkit]"
-		set script "$script [file join $pfmDir pfm.kit] $pfmDir\n"
+		append script "[file join $pfmDir tclkit tclkit]"
+		append script " [file join $pfmDir pfm.kit] $pfmDir\n"
 	    }
 	    default {
 		set script {}
@@ -255,9 +255,9 @@ namespace eval install {
 	}
 	set pgintclLink [file join $pfmDir pgin.tcl]
 	switch $API {
-	    "pgintcl 3.0.0" {
+	    "pgintcl 3.0.1" {
 		file delete $pgintclLink
-		set pgtclTarget [file join $pfmDir "pgintcl-3.0.0" "pgin.tcl"]
+		set pgtclTarget [file join $pfmDir "pgintcl-3.0.1" "pgin.tcl"]
 		if { [catch {file link $pgintclLink $pgtclTarget} errMsg] } then {
 		    set success false
 		    bell
@@ -325,4 +325,4 @@ namespace eval install {
 
     initRootWindow
 
-}
\ No newline at end of file
+}
