@@ -1,12 +1,13 @@
-# $Id: tkpsql,v 1.5 2003-06-30 22:53:35+00 lbayuk Exp $
+# $Id: tkpsql,v 1.12 2006-08-13 21:53:17+00 lbayuk Exp $
 # tkpsql - Interactive PostgreSQL SQL Interface
-# Copyright 2003 by ljb (lbayuk@mindspring.com)
+# Copyright 2003-2006 by ljb (lbayuk@mindspring.com)
 # May be freely distributed with or without modification; must retain this
 # notice; provided with no warranties.
 # See the file COPYING for complete information on usage and redistribution
 # of this file, and for a disclaimer of all warranties.
 
 # Global variables:
+#   version   - Our version string.
 #   widgets() - Main widget pathnames : input output status
 #   n_history - Number of history elements
 #   history() - History array 1:n_history
@@ -14,6 +15,7 @@
 #   history_q - Index in history where next command will be recalled from
 #   db        - Handle to open database, if empty there is no connection.
 #   dbinfo()  - Remembers db conection info: host, user, dbname, port, password
+#     dbinfo(has_schema)   Flag: Database has schemas (PostgreSQL >=7.3)
 #   form_status - Temporary flag for waiting on a popup
 #   pwd       - Starting directory for file open/save
 #   option()  - Array of options
@@ -21,11 +23,14 @@
 #   " (debug)     - Debug flag, 0 for none
 #   " (maxlook)   - Max. result rows to examine for column widths
 #   " (clear)     - Clear output pad before each command results
-#   special()     - SQL for special database queries, index by code,
-#         value is a 2-element list of {sql title}
-#   special_codes - A list of special() indexes, order as they should
+#   special()     - SQL for special database queries, index by code.
+#   special_title() - Titles for special queries, indexed by code.
+#   special_codes - A list of special*() indexes, ordered as they should
 #         be displayed in the popup.
 
+set version 1.2.1
+package require Tk
+
 # ===== Utility Routines =====
 
 # Initialization:
@@ -53,51 +58,195 @@ proc initialize {} {
   dbms_load
   font create monofont -family Courier
   font create boldfont -family Courier -weight bold
-  init_special
 }
 
 # Initialize the array of special database queries.
-# The index values are arbitrary codes.
-# The values are lists of {{SQL Query} {Title}}.
+# This has to be done after connecting to the database, so we know if
+# the schema-aware versions should be used. It can be called again as needed.
+# special(c) contains the SQL for code 'c'.
+# special_title(c) contains the displayed title for code 'c'.
+# The index values 'c' are arbitrary codes.
 # The list special_codes contains the ordered list of indexes.
-# I mostly copied the SQL queries from psql.
+#
+# I mostly copied the SQL queries from psql. The 'schema-aware' queries are
+# based on PostgreSQL-7.3.4; the 'non-schema' versions are from some older
+# version. But in some cases, I took advantage of the special views.
+#
+# Note: The pre-7.3 queries are no longer updated/maintained because I don't
+# have pre-7.3 server to test them on.
+#
 proc init_special {} {
-  global special special_codes
+  global dbinfo special special_title special_codes
+  catch {unset special_codes special_title special}
 
-  set special_codes { dbs tables index user group rights }
+  if {$dbinfo(has_schema)} init_special_new init_special_old
+}
 
-  set special(dbs) \
-    {"select datname as database, usename as owner from pg_database, pg_user\
-        where datdba=usesysid order by datname"\
-     "List Databases" }
+# Initialize special queries for PostgreSQL-7.3 and higher.
+# See comments for init_special
+proc init_special_new {} {
+  global special special_title special_codes
 
-  set special(user) \
-    {"select usename as username, usesysid as id from pg_user\
-        order by usename" \
-     "List Users" }
+  # This is the ordered list of codes whose titles will be displayed.
+  set special_codes { dbs tables views index rules seqs rights user group }
+
+  set special_title(dbs) "List Databases"
+  set special(dbs) {
+    select datname as "Database Name", usename as "Owner"
+    from pg_database, pg_user
+    where datdba=usesysid order by datname
+  }
+
+  set special_title(tables) "List Tables"
+  set special(tables) {
+    select schemaname as "Schema", tablename as "Table", tableowner as "Owner"
+    from pg_catalog.pg_tables
+    where schemaname not in ('pg_catalog', 'pg_toast', 'information_schema')
+    order by 1,2
+  }
+
+  set special_title(views) "List Views"
+  set special(views) {
+    select schemaname as "Schema", viewname as "View", viewowner as "Owner",
+      definition as "Definition"
+    from pg_catalog.pg_views
+    where schemaname not in ('pg_catalog', 'pg_toast', 'information_schema')
+    order by 1,2
+  }
+
+  set special_title(index) "List Indexes"
+  set special(index) {
+    select schemaname as "Schema", indexname as "Index-Name",
+      tablename as "Base-Table", indexdef as "Definition"
+    from pg_catalog.pg_indexes
+    where schemaname not in ('pg_catalog', 'pg_toast', 'information_schema')
+    order by 1,2
+  }
+
+  set special_title(rules) "List Rules"
+  set special(rules) {
+    select schemaname as "Schema", rulename as "Rule",
+      definition as "Definition"
+    from pg_catalog.pg_rules
+    where schemaname not in ('pg_catalog', 'pg_toast', 'information_schema')
+    order by 1,2
+  }
+
+  # Sequences - no special view, so do it manually.
+  set special_title(seqs) "List Sequences"
+  set special(seqs) {
+    select n.nspname as "Schema", c.relname as "Sequence",
+      u.usename as "Owner"
+    from pg_namespace n, pg_class c, pg_user u
+    where n.oid = c.relnamespace and c.relowner = u.usesysid
+      and relkind = 'S'
+      and n.nspname not in ('pg_catalog', 'pg_toast', 'information_schema')
+    order by 1, 2
+  }
+
+  set special_title(rights) "Show Permissions"
+  set special(rights) {
+    select n.nspname as "Schema", c.relname as "Relation",
+        u.usename as "Owner", c.relacl as "Access Control List"
+    from pg_class c, pg_user u, pg_namespace n
+    where c.relowner = u.usesysid and c.relnamespace = n.oid
+      and c.relkind in ('r', 'v', 'S')
+      and pg_catalog.pg_table_is_visible(c.oid)
+      and n.nspname not in ('pg_catalog', 'pg_toast', 'information_schema')
+    order by 1, 2
+  }
+
+  set special_title(user) "List Users"
+  set special(user) {
+    select usename as "Username", usesysid as "User-ID",
+      trim (leading ' ' from
+        trim (trailing ',' from
+           case when usesuper then ' Superuser,' else '' end
+        || case when usecreatedb then ' Create Database,' else '' end
+        || case when usecatupd then ' Update Catalogs,' else '' end))
+      as "Rights"
+    from pg_user order by usename
+  }
  
-  set special(group) \
-    {"select groname as groupname, grosysid as id, grolist as members\
-        from pg_group order by groname" \
-     "List Groups"}
+  set special_title(group) "List Groups"
+  set special(group) {
+    select groname as "Groupname", grosysid as "Group-ID",
+      grolist as "Member-IDs"
+    from pg_group order by groname
+  }
+}
 
-  set special(tables) \
-    {"select usename as username, relname as table, relkind as kind\
-        from pg_class, pg_user where relkind = 'r' and relname !~ '^pg_'\
-        and usesysid=relowner order by relname" \
-     "List Tables"}
+# Initialize special queries for PostgreSQL older than 7.3.
+# See comments for init_special. This is UNMAINTAINED.
+proc init_special_old {} {
+  global special special_title special_codes
 
-  set special(index) \
-    {"select usename as username, relname as name, relkind as kind\
-        from pg_class, pg_user where (relkind='i' or relkind='S') and\
-        relname !~ '^pg_' and usesysid=relowner order by relname" \
-     "List Indexes/Sequences"}
+  # This is the ordered list of codes whose titles will be displayed.
+  set special_codes { dbs tables index rights user group }
 
-  set special(rights) \
-    {"select relname as table, usename as owner, relacl as acl from\
-        pg_class, pg_user where (relkind = 'r' or relkind = 'S') and\
-        relname !~ '^pg_' and usesysid=relowner order by relname" \
-     "Show Table/Sequence Rights"}
+  set special_title(dbs) "List Databases"
+  set special(dbs) {
+    select datname as "Database Name", usename as "Owner"
+    from pg_database, pg_user
+    where datdba=usesysid order by datname
+  }
+
+  set special_title(tables) "List Tables"
+  set special(tables) {
+    select usename as username, relname as table, relkind as kind
+    from pg_class, pg_user where relkind = 'r' and relname !~ '^pg_'
+    and usesysid=relowner order by relname
+  }
+
+  set special_title(index) "List Indexes/Sequences"
+  set special(index) {
+    select usename as username, relname as name, relkind as kind
+        from pg_class, pg_user where (relkind='i' or relkind='S') and
+        relname !~ '^pg_' and usesysid=relowner order by relname
+  }
+
+  set special_title(rights) "Show Table/Sequence Rights"
+  set special(rights) {
+    select relname as table, usename as owner, relacl as acl from
+        pg_class, pg_user where (relkind = 'r' or relkind = 'S') and
+        relname !~ '^pg_' and usesysid=relowner order by relname
+  }
+
+  set special_title(user) "List Users"
+  set special(user) {
+    select usename as "Username", usesysid as "User-ID",
+      usecreatedb as "Create-DB?",
+      usesuper as "Superuser?",
+      usecatupd as "Update-Catalogs?"
+    from pg_user order by usename
+  }
+ 
+  set special_title(group) "List Groups"
+  set special(group) {
+    select groname as "Groupname", grosysid as "Group-ID",
+      grolist as "Member-IDs"
+    from pg_group order by groname
+  }
+}
+
+
+# Initialize after connecting to a database
+# If an error occurs querying the database, ignore the error and don't
+# report it. (Will only report errors from user-issued queries.)
+# This also inializes the special queries.
+proc init_post_connect {} {
+  global db dbinfo
+
+  # Determine if the database supports schemas.
+  set dbinfo(has_schema) 0
+  if {![catch {pg_exec $db "select nspname from pg_namespace limit 1"} r]} {
+    if {[pg_result $r -status] == "PGRES_TUPLES_OK"} {
+      set dbinfo(has_schema) 1
+    }
+    pg_result $r -clear
+  }
+  debug_puts "has_schema: $dbinfo(has_schema)"
+  init_special
 }
 
 # Pluralization
@@ -123,16 +272,55 @@ proc debug_puts {s} {
 
 # Load PostgreSQL support with library or emulator:
 proc dbms_load {} {
-  global tcl_platform
   # If it is already loaded; e.g. running under pgtksh, nothing to do.
   if {[info commands pg_connect] != ""} return
   # Use my pgin.tcl interface library from the same directory:
-  set load_command "source [file join [file dirname [info script]] pgin.tcl]"
-  if {[catch {uplevel #0 $load_command} msg]} {
+  set cmd [list source [file join [file dirname [info script]] pgin.tcl]]
+  if {[catch {uplevel #0 $cmd} msg]} {
     error "Error: Unable to load database support. $msg"
   }
 }
 
+# ===== GUI / Window Utilities =====
+
+# Center a window over another window.
+#   $win : Window to center
+#   $over : What to center it over:
+#     "ROOT" => center over the screen.
+#     "PARENT" => center over $win's parent window.
+#     Otherwise $over is the name of a window to center $win over.
+# On return, the window will be mapped (de-iconified).
+proc center_window {win over} {
+  wm withdraw $win
+  update
+  if {$over == "ROOT"} {
+    set base_x 0
+    set base_y 0
+    set base_w [winfo screenwidth $win]
+    set base_h [winfo screenheight $win]
+  } else {
+    if {$over == "PARENT"} {
+      set overwin [winfo parent $win]
+    } else {
+      set overwin $over
+    }
+    set base_x [winfo rootx $overwin]
+    set base_y [winfo rooty $overwin]
+    set base_w [winfo width $overwin]
+    set base_h [winfo height $overwin]
+  }
+  set win_w [winfo reqwidth $win]
+  set win_h [winfo reqheight $win]
+  if {[set win_x [expr {$base_x + int(($base_w - $win_w) / 2)}]] < 0} {
+    set win_x 0
+  }
+  if {[set win_y [expr {$base_y + int(($base_h - $win_h) / 2)}]] < 0} {
+    set win_y 0
+  }
+  wm geometry $win +$win_x+$win_y
+  wm deiconify $win
+}
+
 # Make a top-level window and return its name:
 proc mk_window {name title} {
   catch {destroy $name}
@@ -142,37 +330,26 @@ proc mk_window {name title} {
   return $name
 }
 
-# Position and Wait for popup window. Taken from DMS but simplified to
-# only handle grabbed window and only. Don't change anything here
-# due to sensitivity to command order on Windows.
+# Position and wait for grabbed popup window.
+# Change with care; MS-Win is very sensitive to the command order.
 proc window_wait {win focus_to varname} {
   global $varname
-
   set save_focus [focus]
-  set parent [winfo toplevel [winfo parent $win]]
-  wm withdraw $win
-  update idletasks
-
-  set parent_root_x [winfo rootx $parent]
-  set parent_width  [winfo width $parent]
-  set win_req_width [winfo reqwidth $win]
-  set x [expr {$parent_root_x + ($parent_width - $win_req_width) / 2}]
-
-  set parent_root_y [winfo rooty $parent]
-  set parent_height [winfo height $parent]
-  set win_req_height [winfo reqheight $win]
-  set y [expr {$parent_root_y + ($parent_height - $win_req_height) / 2}]
-
+  center_window $win PARENT
   focus $focus_to
   grab set $win
-  wm geometry $win +$x+$y
-  update idletasks
-  wm deiconify $win
   tkwait variable $varname
   destroy $win
   catch {focus $save_focus}
 }
 
+# Build a button with key binding(s) and command. Returns widget name.
+proc mk_button {widget label key command} {
+  button $widget -text "$label $key" -command $command
+  bind . $key "$widget invoke"
+  return $widget
+}
+
 # Make a 'buttons' frame with OK and Cancel buttons.
 proc mk_buttons {toplevel {ok_action {set form_status 1}}} {
   set f $toplevel.buttons
@@ -184,6 +361,8 @@ proc mk_buttons {toplevel {ok_action {set form_status 1}}} {
   pack $f.ok $f.cancel -side left -padx 3 -pady 3
 }
 
+# ===== UI Support Routines =====
+
 # Append a line to the output window:
 proc oputs {s {tag ""}} {
   global widgets
@@ -316,11 +495,17 @@ proc run_sql {sql {title ""}} {
   show_status ""
   if {$status == "PGRES_COMMAND_OK"} {
     # Command completed with no tuples (e.g. insert, update, etc.).
-    if {[set oid [pg_result $reply -oid]] != ""} {
-      oputs "OK, OID=$oid" bold
-    } else {
-      oputs "OK" bold
+    # Show the OID, if available. (Not available should be 0, but there was
+    # some confusion early about this and it might be an empty string.)
+    set show OK
+    if {[set oid [pg_result $reply -oid]] != 0 && $oid != ""} {
+      append show ", OID=$oid"
     }
+    # Show affected tuple count. Not all pgtcl's support this.
+    if {![catch {pg_result $reply -cmdTuples} n] && $n != ""} {
+      append show ", $n row[plural $n] affected"
+    }
+    oputs $show bold
     clear_input
   } elseif {$status != "PGRES_TUPLES_OK"} {
     # Generally this will be PGRES_FATAL_ERROR, but any other status
@@ -384,17 +569,17 @@ proc do_connect_done {toplevel} {
   }
   set db $result
   show_status "Connected to database $dbinfo(dbname)@$dbinfo(host)"
+  init_post_connect
   set form_status 1
 }
 
 # Run special queries. See do_special and init_special.
 proc run_special {code} {
-  global form_status special
+  global form_status special special_title
   # Close the special query popup:
   set form_status 1
   update
-  # Pass the SQL and the Title as arguments to run_sql:
-  eval run_sql $special($code)
+  run_sql $special($code) $special_title($code)
 }
 
 # ===== Menu Command Routines =====
@@ -495,7 +680,7 @@ proc do_disconnect {} {
 
 # Load a file into the input window:
 proc do_loadin {} {
-  global widgets pwd tcl_platform
+  global widgets pwd
 
   set fname [tk_getOpenFile -initialdir $pwd -title "Load input window"]
   if {$fname == ""} return
@@ -512,7 +697,7 @@ proc do_loadin {} {
 
 # Save Input or Output text areas to a file.
 proc do_save {which} {
-  global widgets pwd tcl_platform
+  global widgets pwd
 
   set fname [tk_getSaveFile -initialdir $pwd -title "Save $which window"]
   if {$fname == ""} return
@@ -587,14 +772,13 @@ proc do_options {} {
 
 # Special queries. See init_special for the data which drives this.
 proc do_special {} {
-  global form_status special special_codes
+  global form_status special special_title special_codes
 
   set t [mk_window .special "Special Queries"]
   set packme {}
   # Generate all the special query buttons:
   foreach code $special_codes {
-    set title [lindex $special($code) 1]
-    button $t.$code -text $title -command "run_special $code"
+    button $t.$code -text $special_title($code) -command "run_special $code"
     lappend packme $t.$code
   }
   button $t.cancel -text Cancel -command "set form_status 0"
@@ -604,18 +788,11 @@ proc do_special {} {
   window_wait $t $t.cancel form_status
 }
 
-# ===== GUI Routines =====
-
-# Build a button with key binding(s) and command. Returns widget name.
-proc mk_button {widget label key command} {
-  button $widget -text "$label $key" -command $command
-  bind . $key "$widget invoke"
-  return $widget
-}
+# ===== Main Window UI =====
 
 # Build the main user interface:
 proc build_ui {} {
-  global widgets
+  global widgets version
 
   set f .buttons
   frame $f
@@ -684,6 +861,13 @@ proc build_ui {} {
 
   pack .buttons .r -side left -fill both
   pack configure .r -expand 1
+
+  # Main window setup:
+  wm title . "tkpsql $version"
+  wm iconname . tkpsql
+  wm protocol . WM_DELETE_WINDOW do_exit
+  center_window . ROOT
+
   focus $widgets(input)
   # Needed on Windows, for some strange reason:
   raise .
